"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
//  tslint:disable: object-literal-sort-keys no-empty
var internal_1 = require("../internal");
var TERSER_DEFAULTS = {
    cache: true,
    parallel: true,
    terserOptions: {
        compress: {
            comparisons: false,
            ecma: 5,
            inline: 2,
            warnings: false,
        },
        mangle: { safari10: true },
        output: {
            ascii_only: true,
            comments: false,
            ecma: 5,
        },
        parse: {
            // we want terser to parse ecma 8 code. However, we don't want it
            // to apply any minfication steps that turns valid ecma 5 code
            // into invalid ecma 5 code.
            ecma: 8,
        },
    },
};
exports.TIMPLA_DEFAULTS = {
    src: 'src',
    dest: 'public',
    staticFiles: {
        src: 'static',
        dest: './',
        srcOptions: {},
        destOptions: {},
    },
    clean: {
        delOptions: { force: true },
    },
    stylesheets: {
        src: 'stylesheets',
        dest: 'stylesheets',
        srcOptions: {},
        destOptions: {},
        autoprefixerOptions: {},
        cssnanoOptions: {},
        postcssOptions: {},
        sassOptions: { includePaths: ['node_modules'] },
        extensions: ['sass', 'scss', 'css'],
        development: {
            sourceMap: true,
        },
        production: {
            sourceMap: false,
        },
    },
    fonts: {
        src: 'fonts',
        dest: 'fonts',
        srcOptions: {},
        destOptions: {},
        extensions: ['woff2', 'woff', 'eot', 'ttf', 'svg'],
    },
    images: {
        src: 'images',
        dest: 'images',
        srcOptions: {},
        destOptions: {},
        extensions: ['jpg', 'jpeg', 'png', 'svg', 'gif', 'webp'],
    },
    svg: {
        svgstore: {},
        outputName: 'icons.svg',
        src: 'svg',
        dest: 'images',
        srcOptions: {},
        destOptions: {},
    },
    html: {
        src: 'html',
        dest: './',
        srcOptions: {},
        destOptions: {},
    },
    javascripts: {
        src: 'javascripts',
        dest: 'javascripts',
        extensions: ['js', 'jsx', 'ts', 'tsx', 'mjs', 'json'],
        entry: {},
        babelLoaderOptions: {},
        webpackBundleAnalyzerOptions: {
            statsFilename: 'timpla-webpack-stats.json',
            openAnalyzer: true,
        },
        customizeWebpackConfig: function (_a) {
            var webpackConfig = _a.webpackConfig;
            return webpackConfig;
        },
        production: {
            devtool: 'source-map',
            hardSourceOptions: false,
            terserConfig: TERSER_DEFAULTS,
        },
        useTypeScript: true,
    },
    browserSync: {
        cwd: internal_1.TIMPLA_PROCESS.INIT_CWD,
        open: !internal_1.TIMPLA_PROCESS.isTimplaReloaded,
    },
    production: {
        rev: false,
        open: false,
    },
    development: {
        open: false,
        timplaWatch: true,
        devtool: 'cheap-module-eval-source-map',
        disableServerFallback: false,
        eslint: {
            esLintLoaderOptions: {},
        },
        middlewareConfig: {
            webpackDevMiddleware: {},
            webpackHotMiddleware: {},
            connectHistoryApiFallbackMiddleware: {},
        },
        tslint: {
            forkTsCheckerOptions: {},
        },
        webpackHotMiddlewareClient: {
            reload: true,
        },
    },
    watch: {
        gulpWatch: {},
    },
    additionalTasks: {
        development: {},
        production: {},
    },
};
//# sourceMappingURL=TIMPLA_DEFAULTS.js.map