"use strict";
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spread = (this && this.__spread) || function () {
    for (var ar = [], i = 0; i < arguments.length; i++) ar = ar.concat(__read(arguments[i]));
    return ar;
};
Object.defineProperty(exports, "__esModule", { value: true });
var chalk_1 = require("chalk");
var log = require("fancy-log");
var gulp = require("gulp");
var replace = require("gulp-replace");
var merge = require("merge-stream");
var path = require("path");
var internal_1 = require("../internal");
var BASE_JS_TEMPLATES = './init_templates/js';
var EXCLUDE_TS_FILES = '!**/*.ts{,?}*';
var fileNamesOnly = function (name) { return name.split('/').splice(-1)[0]; };
var getJSInitTemplatePath = function () {
    var paths = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        paths[_i] = arguments[_i];
    }
    return path.join.apply(path, __spread([BASE_JS_TEMPLATES], paths));
};
exports.init = function (enableTS) {
    if (enableTS === void 0) { enableTS = true; }
    return function () {
        log(chalk_1.default.green('Generating default Timpla project files'));
        var configFiles = ['.eslintrc', '.prettierrc'];
        var srcFiles = ['src/**/*', 'src/**/.gitkeep'];
        if (enableTS) {
            configFiles.push('babel.config.js', 'tslint.json', 'tsconfig.json');
        }
        else {
            configFiles.push(getJSInitTemplatePath('babel.config.js'));
            srcFiles.push(getJSInitTemplatePath('src', '**/*'), EXCLUDE_TS_FILES);
        }
        var timplaConfigFile = enableTS ? '.timplaconfig.js' : getJSInitTemplatePath('.timplaconfig.js');
        var projectRoot = internal_1.projectPath();
        var projectSrPath = internal_1.projectSrcPath();
        var configStream = gulp.src(configFiles).pipe(gulp.dest(projectRoot));
        var srcStream = gulp.src(srcFiles).pipe(gulp.dest(projectSrPath));
        var timplaConfStream = gulp
            .src(timplaConfigFile)
            .pipe(replace('./lib/public', 'timpla'))
            .pipe(gulp.dest(projectRoot));
        var filesGenerated = __spread(configFiles, [timplaConfigFile]).map(fileNamesOnly);
        var filesGeneratedDesc = filesGenerated.join(', ');
        log(chalk_1.default.green("Config files generated: " + filesGeneratedDesc + "."));
        log(chalk_1.default.yellow("\nTo start the dev server:\n"), chalk_1.default.magenta("\nnpx timpla\n"));
        return merge(timplaConfStream, configStream, srcStream);
    };
};
//# sourceMappingURL=init.js.map